<template>
  <AdminLayout>
    <DashboardHeader>
      <DashboardTitle>{{ $t("app.admin.domains.title") }}</DashboardTitle>
    </DashboardHeader>
    <DashboardBody>
      <ListActions>
        <FiltersSearch v-model="filters.search" />
        <ButtonGroup>
          <FiltersSelect
            v-model="filters"
            @clear="filters.clear"
            @updated="fetchDomains"
          >
            <FormInput id="filter_domain">
              <FormLabel>{{ $t("app.admin.domains.filters.domain") }}</FormLabel>
              <FormText
                v-model="filters.domain"
                name="domain"
              />
            </FormInput>
            <FormInput id="filter_redirect">
              <FormLabel>{{ $t("app.admin.domains.filters.redirect") }}</FormLabel>
              <FormText
                v-model="filters.redirect"
                name="redirect"
              />
            </FormInput>
          </FiltersSelect>
          <RouterLink
            v-if="hasPermission(Permission.ADMIN_DOMAINS_CREATE)"
            :to="{ name: 'admin.domains.create' }"
            class="btn btn-primary"
          >
            <ButtonIcon :icon="PlusRegularIcon">
              {{ $t("app.admin.domains.create.title") }}
            </ButtonIcon>
          </RouterLink>
        </ButtonGroup>
      </ListActions>
      <template v-if="domainsFetch.hasErrors">
        <ErrorState />
      </template>
      <template v-else>
        <ListGroup :is-loading="domainsFetch.isFetching">
          <ListTable>
            <ListTableHeader>
              <ListTableRow>
                <ListTableCell primary>
                  {{ $t("app.admin.domains.form.domain") }}
                </ListTableCell>
                <ListTableCell>
                  {{ $t("app.admin.domains.form.links") }}
                </ListTableCell>
                <ListTableCell>
                  {{ $t("app.admin.domains.form.redirect") }}
                </ListTableCell>
                <ListTableCell>
                  {{ $t("app.admin.domains.form.created_at") }}
                </ListTableCell>
                <ListTableCell>
                  {{ $t("app.admin.domains.form.updated_at") }}
                </ListTableCell>
              </ListTableRow>
            </ListTableHeader>
            <ListTableBody>
              <component
                :is="hasPermission(Permission.ADMIN_DOMAINS_UPDATE) ? ListTableRouterLinkRow : ListTableRow"
                v-for="domain in collection.data"
                :key="domain.id"
                :to="hasPermission(Permission.ADMIN_DOMAINS_UPDATE) ? { name: 'admin.domains.update', params: { id: domain.id } } : undefined"
              >
                <ListTableCell>
                  {{ domain.domain }}
                </ListTableCell>
                <ListTableCell>
                  {{ domain.links_count }}
                </ListTableCell>
                <ListTableCell>
                  {{ domain.redirect || "-" }}
                </ListTableCell>
                <ListTableCell>
                  {{ new Date(domain.created_at).toLocaleString() }}
                </ListTableCell>
                <ListTableCell>
                  {{ new Date(domain.updated_at).toLocaleString() }}
                </ListTableCell>
              </component>
            </ListTableBody>
          </ListTable>
          <ListEmpty
            v-if="collection.empty() && !domainsFetch.isFirthFetch"
            :icon="DomainIllustration"
            description="app.admin.domains.empty.description"
            title="app.admin.domains.empty.title"
          />
        </ListGroup>
        <Pagination
          :collection="collection"
          @change-page="filters.changePage"
        />
      </template>
    </DashboardBody>
  </AdminLayout>
  <RouterView @fetch="fetchDomains()" />
</template>

<script lang="ts" setup>
import DashboardHeader from "@/js/contexts/shared/ui/components/dashboard/DashboardHeader.vue";
import DashboardBody from "@/js/contexts/shared/ui/components/dashboard/DashboardBody.vue";
import AdminLayout from "@/js/contexts/admin/shared/ui/layouts/AdminLayout.vue";
import DashboardTitle from "@/js/contexts/shared/ui/components/dashboard/DashboardTitle.vue";
import useFetch from "@/js/contexts/shared/composables/useFetch";
import useCollection from "@/js/contexts/shared/composables/useCollection";
import useFilters from "@/js/contexts/shared/composables/useFilters";
import type PaginatedCollection from "@/js/contexts/shared/models/PaginatedCollection";
import { pickBy } from "lodash-es";
import type { Domain } from "@/js/contexts/shared/models/Domain";
import FiltersSelect from "@/js/contexts/shared/ui/components/filters/FiltersSelect.vue";
import FiltersSearch from "@/js/contexts/shared/ui/components/filters/FiltersSearch.vue";
import ButtonGroup from "@/js/contexts/shared/ui/components/button/ButtonGroup.vue";
import ListActions from "@/js/contexts/shared/ui/components/list/ListActions.vue";
import FormInput from "@/js/contexts/form/ui/components/FormInput.vue";
import FormText from "@/js/contexts/form/ui/components/FormText.vue";
import FormLabel from "@/js/contexts/form/ui/components/FormLabel.vue";
import { Permission } from "@/js/contexts/shared/models/Permission";
import ButtonIcon from "@/js/contexts/shared/ui/components/button/ButtonIcon.vue";
import useAuth from "@/js/contexts/auth/composables/useAuth";
import PlusRegularIcon from "@/svg/icons/plus-regular.svg?component";
import ErrorState from "@/js/contexts/shared/ui/components/error/ErrorState.vue";
import ListGroup from "@/js/contexts/shared/ui/components/list/ListGroup.vue";
import ListTableRouterLinkRow from "@/js/contexts/shared/ui/components/list/ListTableRouterLinkRow.vue";
import ListTableRow from "@/js/contexts/shared/ui/components/list/ListTableRow.vue";
import ListEmpty from "@/js/contexts/shared/ui/components/list/ListEmpty.vue";
import ListTableHeader from "@/js/contexts/shared/ui/components/list/ListTableHeader.vue";
import ListTableCell from "@/js/contexts/shared/ui/components/list/ListTableCell.vue";
import ListTableBody from "@/js/contexts/shared/ui/components/list/ListTableBody.vue";
import Pagination from "@/js/contexts/shared/ui/components/pagination/Pagination.vue";
import ListTable from "@/js/contexts/shared/ui/components/list/ListTable.vue";
import DomainIllustration from "@/svg/illustrations/domain.svg?component";

const { t } = useI18n();
const { hasPermission } = useAuth();

useHead({
  title: t("app.admin.domains.title"),
});

const domainsFetch = useFetch("/api/admin/domains");
const collection = useCollection<Domain>();
const filters = useFilters(
  {
    search: "",
    domain: "",
    redirect: "",
  },
  ["search"],
);

onMounted(() => {
  fetchDomains();
});

const fetchDomains = () => {
  domainsFetch.get<PaginatedCollection<Domain>>(pickBy(filters.value.data())).then((response) => {
    collection.fromResponse(response.data);
  });
};
</script>
